"""
Copyright (c) 2022 Luke Kreczko. All rights reserved.

xrdsum: Package for retrieving and calculating checksums for XRootD
"""


from __future__ import annotations

from ._version import version as __version__

__all__ = ("__version__",)
